#ifndef GFXSCRIPT_INCLUDED // -*- C++ -*-
#define GFXSCRIPT_INCLUDED
#if !defined(__GNUC__)
#  pragma once
#endif

/************************************************************************

  Simple low-level file format scripting support.
    
  $Id: script.h,v 1.4 2000/10/30 19:48:03 garland Exp $

 ************************************************************************/

#include "gfx.h"

enum {
    SCRIPT_OK = 0,
    SCRIPT_ERR_UNDEF = 1,
    SCRIPT_ERR_SYNTAX = 2,
    SCRIPT_ERR_UNSUPPORTED = 3,
};

typedef int (*script_handler)(const char *name, char *argline);
extern void register_command(const char *name, script_handler proc);

extern int script_do_string(const char *line);
extern int script_do_file(const char *filename);

////////////////////////////////////////////////////////////////////////
//
// Low-level support routines
//
// These procedures provide support for manipulating the command
// strings passed to scripting procedures.
//

extern char *string_trimleft(char *line);
extern char *string_trimright(char *line);
extern char *string_shift_token(char **line);

extern int string_shift_numbers(char **line, double *x, int n);
extern int string_shift_numbers(char **line, float *x, int n);
extern int string_shift_numbers(char **line, int *x, int n);

////////////////////////////////////////////////////////////////////////
//
// "Compile-time" registration mechanism
//
// We want to allow separate code modules to register their scripting
// procedures without the main program having to call intialization
// functions for each one.  So we take advantage of C++, and use
// constructors for global "variables" to accomplish all this
// initialization implicitly.
//
// Recipe for use:
//
// 	static script_defn handlers[] = {
//	    {"sample", sample_proc},
//	};
//
//	REGISTER_COMMANDS(handlers);
//

struct script_defn { const char *name; script_handler proc; };

class __script_registration
{
public:
    __script_registration(script_defn *table, int n)
    {
	for(int i=0; i<n; i++)  register_command(table[i].name, table[i].proc);
    }
};

#define REGISTER_COMMANDS(t) \
	static __script_registration __scripted(t, sizeof(t)/sizeof(t[0]))

// GFXSCRIPT_INCLUDED
#endif
