/************************************************************************

  Handy 3D geometrical primitives

  $Id: geom3d.cxx,v 1.1 2000/05/24 19:31:58 garland Exp $

 ************************************************************************/

#include <gfx/gfx.h>
#include <gfx/geom3d.h>

Vec3 triangle_raw_normal(const Vec3& v1, const Vec3& v2, const Vec3& v3)
{
    Vec3 a = v2 - v1;
    Vec3 b = v3 - v1;
    return a^b;
}

double triangle_area(const Vec3& v1, const Vec3& v2, const Vec3& v3)
{
    return 0.5 * norm(triangle_raw_normal(v1, v2, v3));
}

Vec3 triangle_normal(const Vec3& v1, const Vec3& v2, const Vec3& v3)
{
    Vec3 n = triangle_raw_normal(v1, v2, v3);
    unitize(n);

    return n;
}

Vec4 triangle_plane(const Vec3& v1, const Vec3& v2, const Vec3& v3)
{
    Vec3 n = triangle_normal(v1, v2, v3);

    return Vec4(n, -(n*v1));
}

Vec4 triangle_raw_plane(const Vec3& v1, const Vec3& v2, const Vec3& v3)
{
    Vec3 n = triangle_raw_normal(v1, v2, v3);

    return Vec4(n, -(n*v1));
}

// avoid repeated computation of 4*sqrt(3)
const double FOUR_ROOT3 = 6.928203230275509;

double triangle_compactness(const Vec3& v1, const Vec3& v2, const Vec3& v3)
{
    double L1 = norm2(v2 - v1);
    double L2 = norm2(v3 - v2);
    double L3 = norm2(v1 - v3);

    return FOUR_ROOT3 * triangle_area(v1, v2, v3) / (L1+L2+L3);
}
