/************************************************************************

  Test the libgfx scripting facility.

  by Michael Garland, 2000.
  
  $Id: t-script.cxx,v 1.2 2000/09/06 21:43:48 garland Exp $

 ************************************************************************/

#include <gfx/gfx.h>
#include <gfx/script.h>
#include <gfx/vec3.h>

// usage: add <x>*
//     Adds all numbers listed on the line and prints the result
// 
// usage: avg <x>*
//     Averages all numbers listed on the line and prints the result
//
int proc_add(const char *name, char *argline)
{
    double sum = 0.0;
    double x;
    int count = 0;

    while( string_shift_numbers(&argline, &x, 1)==1 )
    {
	sum += x;
	count++;
    }

    if( !strcmp(name, "avg") && count!=0 )
	sum /= (double)count;

    cout << sum << endl;
    return SCRIPT_OK;
}

// usage: vec3 <x> <y> <z>
//     Constructs a 3-vector and prints the result
int proc_vec3(const char *name, char *argline)
{
    Vec3 v;

    if( string_shift_numbers(&argline, v, 3) != 3 )  return SCRIPT_ERR_SYNTAX;

    cout << v << endl;
    return SCRIPT_OK;
}

// usage: echo <msg>
//     Prints all the text following the command name verbatim
int proc_echo(const char *name, char *argline)
{
    if(argline) cout << argline;
    cout << endl;
    return SCRIPT_OK;
}

static script_defn handlers[] = {
    {"add", proc_add},
    {"avg", proc_add},
    {"echo", proc_echo},
    {"vec3", proc_vec3},
};

REGISTER_COMMANDS(handlers);

main(int argc, char *argv[])
{
    for(int i=1; i<argc; i++)
	script_do_file(argv[i]);
}
