/************************************************************************

  Test various vector math facilities provided by libgfx.

  by Michael Garland, 2000.
  
  $Id: t-vec.cxx,v 1.3 2000/11/03 17:14:49 garland Exp $

 ************************************************************************/

#include <gfx/gfx.h>

#include <gfx/vec2.h>
#include <gfx/vec3.h>
#include <gfx/vec4.h>
#include <gfx/intvec.h>

using namespace std;

void test_intvec()
{
    cout << "Testing IntVec types" << endl;

    class Normal : public IntVec<short, SHRT_MAX, 3>
    {
    public:
	Normal(const Vec3& v) { (*this) = v; }

	Vec3 unpack() const { return Vec3((*this)[0],(*this)[1],(*this)[2]); }
	void pack(const Vec3& v)
	    { set(0, v[0]); set(1, v[1]); set(2, v[2]); }

	Normal& operator=(const Vec3& v) { pack(v); return *this; }
    };

    Normal n = Vec3(1.0, 0.4, -1.0);
    cout << "  n = " << n.unpack() << endl;

    n.set(0, -1.0);  n.set(1, 0.4);  n.set(2, 1);
    cout << "  n = " << n[0] << " " << n[1] << " " << n[2] << endl;


    typedef IntVec3<unsigned char, UCHAR_MAX> byteColor;
    byteColor rgb(0.8, 0.2, 0.2);
    cout << "  rgb = " << rgb.unpack() << endl;
}

void test_vec2()
{
    cout << "+ Testing class Vec2" << endl;

    Vec2 u(1, 0);
    Vec2 v(0, 1);

    Vec2 x = u * 2;
    Vec2 y = v / 2;



    cout << "  x = " << x << endl;
    cout << "  y = " << y << endl;
}

void test_vec3()
{
    cout << "+ Testing class Vec3" << endl;

    Vec3 u(1, 0, 0);
    Vec3 v(0, 1, 0);

    Vec3 x = u * 2;
    Vec3 y = v / 2;

    cout << "  x = " << x << endl;
    cout << "  y = " << y << endl;

    Vec3 a(0.8, 0.0, 0.0);
    Vec3 b = 1*a;
    cout << "  b = " << b << endl;
}

void test_vec4()
{
    Vec4 u(1, 0, 0, 0);
    Vec4 v(0, 1, 0, 0);

    Vec4 x = u * 2;
    Vec4 y = v / 2;

    cout << "  x = " << x << endl;
    cout << "  y = " << y << endl;
}


main()
{
    test_vec2();
    test_vec3();
    test_vec4();

    test_intvec();

    return 0;
}
